@@ -3,10 +3,14 @@ using GLib;
 using Gtk;
 using GtkSharp;
 using System;
+using Mono.Unix;
+using FSpot.Xmp;
 
 public class PhotoView : EventBox {
 	FSpot.Delay description_delay; 
 
+	private bool has_selection = false;
+
 	public FSpot.PhotoImageView View {
 		get {
 			return photo_view;
@@ -46,11 +50,12 @@ public class PhotoView : EventBox {
 	private Gtk.Button crop_button;
 	private Gtk.Button redeye_button;
 	private Gtk.Button color_button;
+	
+	Gtk.Button desaturate_button;
+	Gtk.Button sepia_button;
 
 	FSpot.AsyncPixbufLoader loader = new FSpot.AsyncPixbufLoader ();
 
-	private const double MAX_ZOOM = 5.0;
-
 	public double Zoom {
 		get {
 			return photo_view.Zoom;
@@ -60,6 +65,16 @@ public class PhotoView : EventBox {
 			photo_view.Zoom = value;
 		}
 	}
+	
+	public double NormalizedZoom {
+		get {
+			return photo_view.NormalizedZoom;
+		}
+
+		set {
+			photo_view.NormalizedZoom = value;
+		}
+	}
 
 
 	// Public events.
@@ -144,9 +159,21 @@ public class PhotoView : EventBox {
 		display_previous_button.Sensitive = prev;
 		display_next_button.Sensitive = next;
 
+		if (valid) {
+			if (has_selection) {
+				tips.SetTip (crop_button, Catalog.GetString ("Crop photo to selected area"), "");
+				tips.SetTip (redeye_button, Catalog.GetString ("Remove redeye from selected area"), "");
+			} else {
+				tips.SetTip (crop_button, Catalog.GetString ("Select an area to crop"), null);
+				tips.SetTip (redeye_button, Catalog.GetString ("Select an area to remove redeye"), null);
+			}
+		}
+
 		crop_button.Sensitive = valid;
 		redeye_button.Sensitive = valid;
 		color_button.Sensitive = valid;
+		desaturate_button.Sensitive = valid;
+		sepia_button.Sensitive = valid;
 	}
 
 	private void UpdateCountLabel ()
@@ -175,10 +202,11 @@ public class PhotoView : EventBox {
 			description_entry.Sensitive = false;
 			description_entry.Text = "";
 		}
-		if (description_entry.Text == "")
-			tips.SetTip (description_entry, description_entry.Text, "This is a tip");
+
+		if (description_entry.Text != "")
+			tips.SetTip (description_entry, description_entry.Text, description_entry.Text);
 		else
-			tips.SetTip (description_entry, null, "This is a tip");
+			tips.SetTip (description_entry, null, null);
 
 		description_entry.Changed += HandleDescriptionChanged;
 	}    
@@ -196,16 +224,33 @@ public class PhotoView : EventBox {
 			UpdateFinished (this);
 	}
 
+	public void ZoomIn ()
+	{
+		photo_view.ZoomIn ();
+	}
+	
+	public void ZoomOut ()
+	{
+		photo_view.ZoomOut ();
+	}
+
 	// Event handlers.
 	private void HandleButtonPressEvent (object sender, ButtonPressEventArgs args)
 	{
 		if (args.Event.Type == EventType.ButtonPress
 		    && args.Event.Button == 3) {
 			PhotoPopup popup = new PhotoPopup ();
-			popup.Activate (args.Event);
+			popup.Activate (this.Toplevel, args.Event);
 		}
 	}
 
+	protected override bool OnPopupMenu ()
+	{
+		PhotoPopup popup = new PhotoPopup ();
+		popup.Activate (this.Toplevel);
+		return true;
+	}
+
 	private void HandleDisplayNextButtonClicked (object sender, EventArgs args)
 	{
 		View.Item.MoveNext ();
@@ -226,6 +271,37 @@ public class PhotoView : EventBox {
 	{
 		ProcessImage (false);
 	}
+	
+	private void ShowError (System.Exception e, FSpot.IBrowsableItem photo)
+	{
+		Dialog md = new FSpot.EditExceptionDialog ((Gtk.Window)this.Toplevel, e, photo); 
+		md.Run ();
+		md.Destroy ();
+	}
+
+	private void HandleSepiaButtonClicked (object sender, EventArgs args)
+	{
+		try {
+			FSpot.SepiaTone sepia = new FSpot.SepiaTone ((Photo)View.Item.Current);
+			sepia.Pixbuf = View.CompletePixbuf ();
+			sepia.Adjust ();
+			query.Commit (Item.Index);
+		} catch (System.Exception e) {
+			ShowError (e, (Photo)View.Item.Current); 
+		}
+	}
+
+	private void HandleDesaturateButtonClicked (object sender, EventArgs args)
+	{
+		try {
+			FSpot.Desaturate desaturate = new FSpot.Desaturate ((Photo) View.Item.Current);
+			desaturate.Pixbuf = View.CompletePixbuf ();
+			desaturate.Adjust ();
+			query.Commit (Item.Index);
+		} catch (System.Exception e) {
+			ShowError (e, (Photo)View.Item.Current);
+		}
+	}
 
 	// FIXME this design sucks, I'm just doing it this way while
 	// I redesign the editing system.
@@ -234,7 +310,7 @@ public class PhotoView : EventBox {
 		int x, y, width, height;
 		if (! photo_view.GetSelection (out x, out y, out width, out height)) {
 			string msg = Mono.Posix.Catalog.GetString ("No selection available");
-			string desc = Mono.Posix.Catalog.GetString ("This tool requires an active selection. Please select a region of the image and try the operation again");
+			string desc = Mono.Posix.Catalog.GetString ("This tool requires an active selection. Please select a region of the photo and try the operation again");
 			
 			HigMessageDialog md = new HigMessageDialog ((Gtk.Window)this.Toplevel, DialogFlags.DestroyWithParent, 
 								    Gtk.MessageType.Error, ButtonsType.Ok, 
@@ -245,66 +321,45 @@ public class PhotoView : EventBox {
 			md.Destroy ();
 			return;
 		}		
-
-		Pixbuf original_pixbuf = photo_view.CompletePixbuf ();
-		if (original_pixbuf == null) {
-			return;
-		}
-
+		
 		Photo photo = (Photo)Item.Current;
-		Exif.ExifData exif_data = new Exif.ExifData (photo.DefaultVersionPath);
-
-		Pixbuf edited;
-		if (redeye) {
-			Gdk.Rectangle area = new Gdk.Rectangle (x, y, width, height);
-			edited = PixbufUtils.RemoveRedeye (original_pixbuf, 
-							   area);
-
-
-		} else { // Crop (I told you it was ugly)
-			edited = new Pixbuf (original_pixbuf.Colorspace, 
-					     false, original_pixbuf.BitsPerSample,
-					     width, height);
-			
-			original_pixbuf.CopyArea (x, y, width, height, edited, 0, 0);
-		}
-
-
-		// FIXME the fact that the selection doesn't go away is a bug in ImageView, it should
-		// be fixed there.
-		photo_view.Pixbuf = edited;
-		photo_view.UnsetSelection ();
-		bool version = false;
-
 		try {
-			if (photo.DefaultVersionId == Photo.OriginalVersionId) {
-				photo.DefaultVersionId = photo.CreateDefaultModifiedVersion (photo.DefaultVersionId, false);
-				version = true;
-				PixbufUtils.SaveJpeg (edited, photo.DefaultVersionPath, 
-						      95, exif_data);
-				FSpot.ThumbnailGenerator.Create (photo.DefaultVersionPath).Dispose ();
-				query.Commit (Item.Index);
-			} else {
-				// FIXME we need to invalidate the thumbnail in the cache as well
-				PixbufUtils.SaveJpeg (edited, photo.DefaultVersionPath, 
-						      95, exif_data);
-				FSpot.ThumbnailGenerator.Create (photo.DefaultVersionPath).Dispose ();
-				query.MarkChanged (Item.Index);
+			if (! System.IO.File.Exists (photo.DefaultVersionPath)) {
+				throw new System.IO.FileNotFoundException ();
 			}
+
+			Pixbuf original_pixbuf = photo_view.CompletePixbuf ();
+
+			if (original_pixbuf == null) {
+				return;
+			}
+			
+			Pixbuf edited;
+
+			if (redeye) {
+				Gdk.Rectangle area = new Gdk.Rectangle (x, y, width, height);
+				edited = PixbufUtils.RemoveRedeye (original_pixbuf, 
+								   area);
+			} else { // Crop (I told you it was ugly)
+				edited = new Pixbuf (original_pixbuf.Colorspace, 
+						     original_pixbuf.HasAlpha, original_pixbuf.BitsPerSample,
+						     width, height);
+				
+				original_pixbuf.CopyArea (x, y, width, height, edited, 0, 0);
+			}
+			
+			
+			// FIXME the fact that the selection doesn't go away is a bug in ImageView, it should
+			// be fixed there.
+			photo_view.UnsetSelection ();
+			
+			bool create_version = photo.DefaultVersionId == Photo.OriginalVersionId;
+			photo.SaveVersion (edited, create_version);
+			photo_view.Pixbuf = edited;
+			query.Commit (Item.Index);
+			query.MarkChanged (Item.Index);
 		} catch (System.Exception e) {
-			string msg = Mono.Posix.Catalog.GetString ("Error editing photo");
-			string desc = String.Format (Mono.Posix.Catalog.GetString ("Received exception \"{0}\". Unable to save image {1}"),
-						     e.Message, photo.Name);
-
-			HigMessageDialog md = new HigMessageDialog ((Gtk.Window)this.Toplevel, DialogFlags.DestroyWithParent, 
-								    Gtk.MessageType.Error, ButtonsType.Ok, 
-								    msg,
-								    desc);
-			md.Run ();
-			md.Destroy ();
-
-			if (version)
-				photo.DeleteVersion (photo.DefaultVersionId);
+			ShowError (e, photo);
 		}
 		
 		photo_view.Fit = true;
@@ -313,7 +368,8 @@ public class PhotoView : EventBox {
 			PhotoChanged (this);
 	}
 
-	private void HandleColorButtonClicked (object sender, EventArgs args) {
+	private void HandleColorButtonClicked (object sender, EventArgs args) 
+	{
 		new FSpot.ColorDialog (photo_view);
 	}	
 
@@ -322,20 +378,7 @@ public class PhotoView : EventBox {
 	{
 		if (description_delay.IsPending) {
 			description_delay.Stop ();
-
-#if true // UPDATE_EXIF_DESCRIPTION
-			Photo photo = query.Photos [description_photo];
-			FSpot.ImageFile img = FSpot.ImageFile.Create (photo.DefaultVersionPath);
-			if (img is FSpot.JpegFile) {
-				FSpot.JpegFile jimg = img as FSpot.JpegFile;
-				jimg.SetDescription (photo.Description);
-				jimg.SaveMetaData (photo.DefaultVersionPath);
-			}
-			//Query.Store.Commit (photo);
 			Query.Commit (description_photo);
-#else
-			Query.Commit (description_photo);
-#endif
 		}
 		return true;
 	}
@@ -381,6 +424,16 @@ public class PhotoView : EventBox {
 			PhotoChanged (this);
 	}
 
+	private void HandleSelectionChanged ()
+	{
+		int x, y, width, height;
+		bool old = has_selection;
+		has_selection = photo_view.GetSelection (out x, out y, out width, out height);
+	
+		if (has_selection != old)
+			UpdateButtonSensitivity ();
+	}
+
 	private void HandleDestroy (object sender, System.EventArgs args)
 	{
 		CommitPendingChanges ();
@@ -413,6 +466,7 @@ public class PhotoView : EventBox {
 		
 		photo_view = new FSpot.PhotoImageView (query);
 		photo_view.PhotoChanged += HandlePhotoChanged;
+		photo_view.SelectionChanged += HandleSelectionChanged;
 
 		ScrolledWindow photo_view_scrolled = new ScrolledWindow (null, null);
 
@@ -431,9 +485,14 @@ public class PhotoView : EventBox {
 		HBox inner_hbox = new HBox (false, 2);
 		//inner_hbox.BorderWidth = 6;
 
-		tag_view = new TagView ();
-		inner_hbox.PackStart (tag_view, false, true, 0);
+		EventBox tag_view_box = new EventBox ();
+		tag_view = new TagView (tag_view_box);
+		tag_view_box.Add (tag_view);
+		inner_hbox.PackStart (tag_view_box, false, true, 0);
+		FSpot.Global.ModifyColors (tag_view_box);
 
+		Label comment = new Label (Catalog.GetString ("Comment:"));
+		inner_hbox.PackStart (comment, false, false, 0);
 		description_entry = new Entry ();
 		inner_hbox.PackStart (description_entry, true, true, 0);
 		description_entry.Changed += HandleDescriptionChanged;
@@ -458,11 +517,21 @@ public class PhotoView : EventBox {
 		redeye_button.Clicked += new EventHandler (HandleRedEyeButtonClicked);
 
 		color_button = new ToolbarButton ();
-		color_button.Add (new Gtk.Image ("f-spot-edit-image", IconSize.Button));
+		color_button.Add (new Gtk.Image ("f-spot-adjust-colors", IconSize.Button));
 		toolbar_hbox.PackStart (color_button, false, true, 0);
 	
 		color_button.Clicked += new EventHandler (HandleColorButtonClicked);
 
+		desaturate_button = new ToolbarButton ();
+		desaturate_button.Add (new Gtk.Image ("f-spot-desaturate", IconSize.Button));
+		toolbar_hbox.PackStart (desaturate_button, false, true, 0);
+		desaturate_button.Clicked += HandleDesaturateButtonClicked;
+
+		sepia_button = new ToolbarButton ();
+		sepia_button.Add (new Gtk.Image ("f-spot-sepia", IconSize.Button));
+		toolbar_hbox.PackStart (sepia_button, false, true, 0);
+		sepia_button.Clicked += HandleSepiaButtonClicked;
+
 		/* Spacer Label */
 		toolbar_hbox.PackStart (new Label (""), true, true, 0);
 
@@ -487,13 +556,12 @@ public class PhotoView : EventBox {
 		UpdateButtonSensitivity ();
 
 		vbox.ShowAll ();
-		tips.SetTip (crop_button, Mono.Posix.Catalog.GetString ("Crop image to selected area"), "so is this one");
-		tips.SetTip (redeye_button, Mono.Posix.Catalog.GetString ("Remove redeye from selected area"), "this is one too");
-		tips.SetTip (color_button, Mono.Posix.Catalog.GetString ("Adjust the image colors"), "This is a tip");
-		tips.SetTip (constraints_option_menu, Mono.Posix.Catalog.GetString ("Constrain the aspect ratio of the selection"), "This is a tip");
-		tips.SetTip (display_next_button, Mono.Posix.Catalog.GetString ("Next image"), "This is a tip");
-		tips.SetTip (display_previous_button, Mono.Posix.Catalog.GetString ("Previous image"), "This is a tip");
-
+		tips.SetTip (color_button, Mono.Posix.Catalog.GetString ("Adjust the photo colors"), "");
+		tips.SetTip (constraints_option_menu, Mono.Posix.Catalog.GetString ("Constrain the aspect ratio of the selection"), "");
+		tips.SetTip (display_next_button, Mono.Posix.Catalog.GetString ("Next photo"), "");
+		tips.SetTip (display_previous_button, Mono.Posix.Catalog.GetString ("Previous photo"), "");
+		tips.SetTip (desaturate_button, Mono.Posix.Catalog.GetString ("Convert the photo to black and white"), "");
+		tips.SetTip (sepia_button, Mono.Posix.Catalog.GetString ("Convert the photo to sepia tones"), "");
 	}
 
 }
