@@ -105,7 +105,7 @@ public class TagCommands {
 				}
 			}
 		}
-		public bool Execute (TagType type, Tag [] selection)
+		public Tag Execute (TagType type, Tag [] selection)
 		{
 			this.CreateDialog ("create_tag_dialog");
 
@@ -120,7 +120,7 @@ public class TagCommands {
 			this.Dialog.DefaultResponse = ResponseType.Ok;
 
 			this.Dialog.Title = Mono.Posix.Catalog.GetString ("Create New Tag");
-			prompt_label.Text = Mono.Posix.Catalog.GetString ("Name of new tag:");
+			prompt_label.Text = Mono.Posix.Catalog.GetString ("Name of New Tag:");
 
 			PopulateCategoryOptionMenu ();
 			this.Category = default_category;
@@ -129,17 +129,15 @@ public class TagCommands {
 
 			ResponseType response = (ResponseType) this.Dialog.Run ();
 
-			bool success = false;
-
+			Tag new_tag = null;
 			if (response == ResponseType.Ok) {
 				try {
 					Category parent_category = Category;
 
 					if (type == TagType.Category)
-						tag_store.CreateCategory (parent_category, tag_name_entry.Text);
+						new_tag = tag_store.CreateCategory (parent_category, tag_name_entry.Text) as Tag;
 					else
-						tag_store.CreateTag (parent_category, tag_name_entry.Text);
-					success = true;
+						new_tag = tag_store.CreateTag (parent_category, tag_name_entry.Text);
 				} catch (Exception ex) {
 					// FIXME error dialog.
 					Console.WriteLine ("error {0}", ex);
@@ -147,7 +145,7 @@ public class TagCommands {
 			}
 
 			this.Dialog.Destroy ();
-			return success;
+			return new_tag;
 		}
 
 		public Create (TagStore tag_store, Gtk.Window parent_window)
@@ -251,7 +249,7 @@ public class TagCommands {
 				     parent = parent.Category)
 					label_builder.Append ("  ");
 				
-				label_builder.Append (category.Name);
+				label_builder.Append (category.Name.Replace ("_", "__"));
 				
 				// FIXME escape underscores.
 				MenuItem item = new MenuItem (label_builder.ToString ());
@@ -273,7 +271,7 @@ public class TagCommands {
 			this.Dialog.DefaultResponse = ResponseType.Ok;
 
 			this.Dialog.Title = Mono.Posix.Catalog.GetString ("Edit Tag");
-			prompt_label.Text = Mono.Posix.Catalog.GetString ("Tag name:");
+			prompt_label.Text = Mono.Posix.Catalog.GetString ("Tag Name:");
 
 			orig_name = last_valid_name = t.Name;
 			tag_name_entry.Text = t.Name;
